<?php

/**
 * Created by PhpStorm.
 * User: David
 * Date: 10/25/2016
 * Time: 9:14 AM
 */
defined( 'ABSPATH' ) or die( "Cannot access pages directly." );

require_once( dirname( __FILE__ ) . '/cms-bb-conversion.php' );
define('BASE_URL', 'http://www.whoi.edu/');

class media_utils {

//load the images into WordPress page, add them to the bottom of the page content or sidebar
//takes WP Page object, array of image metadata (title, url)
	public static function load_images( $the_page, $image_array, $use_sidebar= false ) {
		if ( isset( $image_array ) ) {
			$image_content = "";
			$page_content = $the_page->post_content;
			foreach ( $image_array as $cms_image ) {
				echo "<br />image: " . $cms_image["title"];
				$cms_path = $cms_image["cmsPath"];
				$title    = $cms_image["title"];

				$desc     = $cms_image["caption"];

				//	$imported_image = media_sideload_image( $url, $the_page->ID, $title );
				$url_prefix = "(http|https):\/\/www\.whoi\.edu";
				$url        = $url_prefix . $cms_path;
				$image_id   = self::media_sideload_image_get_id( "http://www.whoi.edu" . $cms_path, $the_page->ID, $title, $desc );

				$src = wp_get_attachment_url( $image_id );
				//replace any image urls found in text with wordpress links
				$page_content = preg_replace( "(" . $url . ")", $src, $page_content, - 1, $count );
				//if no url replacements were found it likely means that images were originally not embedded in web page text,
				//so we'll add them to the bottom of the page or to the side div
				if ( $count == 0 ) {
					$image_content = $image_content . "<img src=\"" . $src . "\"/>";
					if ( isset( $desc ) ) {
						$image_content = $image_content . "<p>" . $desc . "</p>";
					}
				}
			}
			if ( $use_sidebar ) {
				cms_bb_conversion::add_text_content($the_page->ID, $image_content, "aside");
			} else {
				//$page_content = $the_page->post_content;
				$page_content = $page_content.$image_content;
				$the_post     = array(
					'ID'           => $the_page->ID,
					'post_content' => $page_content
				);
				wp_update_post( $the_post );
			}
		}

	}

	public static function load_files( $the_page, $file_array, $use_sidebar= false ) {
		if ( isset( $file_array ) ) {
			$files = $file_array;
			echo "<br />about to load files...";
			$file_content = "";
			$desc = "";
			$title = "";
			$page_content = $the_page->post_content;
			foreach ( $files as $cmsFile ) {
				$cms_file_path     = BASE_URL . $cmsFile["cmsPath"];
				$cms_url           = $cmsFile["cmsUrl"];
				if (isset($cmsFile["title"])) {
					$title = $cmsFile["title"];
				}
				if (isset($cmsFile["caption"])) {
					$desc = $cmsFile["caption"];
				}
				$imported_file_url = media_utils::upload_file( $cms_file_path,  $the_page->ID, $title );
				if ( is_wp_error( $imported_file_url ) ) {
					echo "error" . $imported_file_url->get_error_message();
				} else {
					/*Check and replace inline links to files with WordPress urls.
					There are some WHOI CMS specific urls like /fileserver.do that will not work
					if the CMS is turned off.  This prevents a future situation where links to files will break.*/

					if ( isset( $imported_file_url ) ) {
						$urls_to_replace = Array();
						//match http, https or relative urls
						$url_prefix      = "(http|https):\/\/www\.whoi\.edu";


						if ( isset( $cms_file_path ) ) {
							//replace CMS doc root in content
							$cms_file_path = str_replace( "/", "\\/", $cms_file_path );
							$cms_file_path = "(".$url_prefix . $cms_file_path . ")";
							array_push( $urls_to_replace, $cms_file_path );
						}
						if ( isset( $cms_url ) ) {
							//replace CMS custom urls in content
							$cms_url = "(". $cms_url . ")";
							array_push( $urls_to_replace, $cms_url );
							$cms_url = "(".$url_prefix . $cms_url . ")";
							array_push( $urls_to_replace, $cms_url );
						}
						$callout_image = NULL;
						if (isset ($cmsFile['calloutPath'])) {
							$image_path = BASE_URL.$cmsFile['calloutPath'];
							$callout_image = media_sideload_image( $image_path, $the_page->ID );
						}
						$count = 0;
						if ( count( $urls_to_replace ) > 0 ) {
							$page_content = str_replace( "&amp;", "&", $page_content );
							$page_content = preg_replace( $urls_to_replace, $imported_file_url, $page_content, -1, $count );
							if (isset ($cmsFile['calloutPath'])) {
								$page_content = str_replace( $cmsFile['calloutPath'], $callout_image, $file_content );
							}

						} //if no url replacements were found it likely means that images were originally not embedded in web page text,
						//so we'll add them to the bottom of the page or to the side div
						if ( $count == 0 ) {

							if (isset($callout_image)) {
								$file_content = $file_content . $callout_image;
							}
							if (!empty($title)) {
								$file_content = $file_content . "<h3><a href=\"" . $imported_file_url . "\">" . $title . "</a></h3>";
							}
							if ( !empty( $desc ) ) {
								$file_content = $file_content . "<p>" . $desc . "</p>";
							}
						}

					}
				}

			}
			//update the page with revised content
			if ( $use_sidebar ) {
				cms_bb_conversion::add_text_content($the_page->ID, $file_content, "aside");
			} else {
				$page_content = $the_page->post_content;
				$page_content = $page_content.$file_content;
				$the_post     = array(
					'ID'           => $the_page->ID,
					'post_content' => $page_content
				);
				wp_update_post( $the_post );
			}
		}

	}

//create a Real Media Libraries collection for a page
	public static function build_rml_collection( $page_name, $parent = - 1, $type = 1 ) {
		if ( current_user_can( 'administrator' ) ) {  //make sure logged in user has admin rights
			$result = wp_rml_create( $page_name, $parent, $type );
			if ( is_wp_error( $result ) ) {
				echo "error" . $result->get_error_message();

				return - 1;
			} else {
				return $result;
			}
		}
	}

//create image galleries using Real Media Libraries
	public static function build_rml_galleries( $the_page, $galleries ) {
		try {
			if ( current_user_can( 'administrator' ) ) {  //make sure logged in user has admin rights
				$collection_id = self::build_rml_collection( $the_page->post_title );
				$image_ids     = Array();
				if ( $collection_id > - 1 ) {
					foreach ( $galleries as $gallery ) {
						$rml_gallery = wp_rml_create( $gallery["title"], $collection_id, 2 );
						$gallery_images = $gallery["images"];

						foreach ( $gallery_images as $cms_image ) {
							//echo "<br />cms image " . $cms_image["title"];
							$url            = "http://www.whoi.edu/" . $cms_image["cmsPath"];
							$desc           = $cms_image["title"];
							$imported_image = self::media_sideload_image_get_id( $url, $the_page->ID, $desc );
							array_push( $image_ids, $imported_image );
						}
						$move = wp_rml_move( $rml_gallery, $image_ids, true );
					}

				}

				if ( is_array( $move ) ) {
					return new WP_Error( 'building image gallery failed' );
				} else {
					return $collection_id;
				}
			}
		} catch (Exception $e) {
			echo 'Exception in build galleries: ', $e->getMessage(), "\n";
		}
	}

	/**
	 * Downloads an image from the specified URL and attaches it to a post.
	 * Modified (simplified) from media_sideload_image in /wp-admin/includes/media.php to return
	 * simply a post id rather than a url or html
	 *
	 * @param string $file The URL of the image to download.
	 * @param int $post_id The post ID the media is to be associated with.
	 * @param string $desc Optional. Description of the image.
	 *
	 * @return string|WP_Error Image post id on success, WP_Error object otherwise.
	 */
	public static function media_sideload_image_get_id( $file, $post_id, $title = null, $desc = null ) {
		if ( ! empty( $file ) ) {
			// Set variables for storage, fix file filename for query strings.
			preg_match( '/[^\?]+\.(jpe?g|jpe|gif|png)\b/i', $file, $matches );
			if ( ! $matches ) {
				return new WP_Error( 'image_sideload_failed', __( 'Invalid image URL' ) );
			}

			$file_array         = array();
			$file_array['name'] = basename( $matches[0] );

			// Download file to temp location.
			$file_array['tmp_name'] = download_url( $file );

			// If error storing temporarily, return the error.
			if ( is_wp_error( $file_array['tmp_name'] ) ) {

				return $file_array['tmp_name'];
			}

			// Do the validation and storage stuff.
			$id = media_handle_sideload( $file_array, $post_id, $desc );

			// update image title and caption (media_handle_sideload doesn't seem to be doing this)
			$image_meta = array(
				'ID'		=> $id,
				'post_title'	=> $title,
				'post_excerpt'	=> $desc,
				'post_content'	=> $desc,
			);

			// Set the image Alt-Text
			update_post_meta( $id, '_wp_attachment_image_alt', $title );

			// Set the image meta (e.g. Title, Excerpt, Content)
			wp_update_post( $image_meta );

			// If error storing permanently, unlink.
			if ( is_wp_error( $id ) ) {
				@unlink( $file_array['tmp_name'] );

				return new WP_Error( 'image_sideload_failed' );
			} else {
				return $id;
			}


		}

	}


	public static function upload_file( $url, $post_id, $desc ) {

		/*
		 * Build the $file_array with
		 * $url = the url of the image
		 * $temp = storing the image in wordpress
		 */
		try {

			$tmp = download_url( $url );

			$file_array = array(
				'name'     => basename( $url ),
				'tmp_name' => $tmp
			);

			/**
			 * Check for download errors
			 * if there are error unlink the temp file name
			 */
			if ( is_wp_error( $tmp ) ) {
				@unlink( $file_array['tmp_name'] );

				return $tmp;
			}

			/**
			 * now we can actually use media_handle_sideload
			 * we pass it the file array of the file to handle
			 * and the post id of the post to attach it to
			 * $post_id can be set to '0' to not attach it to any particular post
			 */

			$id = media_handle_sideload( $file_array, $post_id, $desc );
			//	echo "<br />File Id: " . $id . " URL: " . $url;
			/**
			 * We don't want to pass something to $id
			 * if there were upload errors.
			 * So this checks for errors
			 */
			if ( is_wp_error( $id ) ) {
				@unlink( $file_array['tmp_name'] );
				echo "<br />Error Id: " . $id->get_error_message() . " Failed to upload file: " . $file_array['tmp_name'];
				return $id;
			}

			/**
			 * No we can get the url of the sideloaded file
			 * $value now contains the file url in WordPress
			 * $id is the attachment id
			 */
			$value = wp_get_attachment_url( $id );

			return $value;
		} catch ( Exception $e ) {
			echo 'Caught exception: ', $e->getMessage(), "\n";
		}


// Now you can do something with $value (or $id)
	}

}